
--[[
Name: MD5-1.0
Revision: $Rev: 25931 $
Author(s): Jean-Claude Wippler, Usz
SVN: http://svn.wowace.com/root/trunk/MD5Lib/MD5-1.0
Description: MD5 (Message-Digest algorithm 5) implementation.
Dependencies: AceLibrary
]]

local major, minor = "MDFive-1.0", "$Revision: 25931 $"

if not AceLibrary then error(major .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(major, minor) then return end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

local lib = {}

local ff = tonumber('ffffffff', 16)
local consts = {}

string.gsub([[
	d76aa478 e8c7b756 242070db c1bdceee
	f57c0faf 4787c62a a8304613 fd469501
	698098d8 8b44f7af ffff5bb1 895cd7be
	6b901122 fd987193 a679438e 49b40821
	f61e2562 c040b340 265e5a51 e9b6c7aa
	d62f105d 02441453 d8a1e681 e7d3fbc8
	21e1cde6 c33707d6 f4d50d87 455a14ed
	a9e3e905 fcefa3f8 676f02d9 8d2a4c8a
	fffa3942 8771f681 6d9d6122 fde5380c
	a4beea44 4bdecfa9 f6bb4b60 bebfbc70
	289b7ec6 eaa127fa d4ef3085 04881d05
	d9d4d039 e6db99e5 1fa27cf8 c4ac5665
	f4292244 432aff97 ab9423a7 fc93a039
	655b59c3 8f0ccc92 ffeff47d 85845dd1
	6fa87e4f fe2ce6e0 a3014314 4e0811a1
	f7537e82 bd3af235 2ad7d2bb eb86d391
	67452301 efcdab89 98badcfe 10325476
]], '(%w+)', function (s) table.insert(consts, tonumber(s, 16)) end)

local function MD5Transform(X, A, B, C, D)
	local f = function(x, y, z) return bit.bor(bit.band(x, y),bit.band(-x - 1, z)) end
	local g = function(x, y, z) return bit.bor(bit.band(x, z),bit.band(y, -z - 1)) end
	local h = function(x, y, z) return bit.bxor(x, bit.bxor(y, z)) end
	local i = function(x, y, z) return bit.bxor(y, bit.bor(x, -z - 1)) end

	local z = function(f, a, b, c, d, x, s, ac)
		a = bit.band(a + f(b, c, d) + x + ac, ff)
		return bit.bor(bit.lshift(bit.band(a, bit.rshift(ff, s)), s),bit.rshift(a, 32 - s)) + b
	end

	local a, b, c, d = A, B, C, D

	a = z(f, a, b, c, d, X[ 0],  7, consts[ 1])
	d = z(f, d, a, b, c, X[ 1], 12, consts[ 2])
	c = z(f, c, d, a, b, X[ 2], 17, consts[ 3])
	b = z(f, b, c, d, a, X[ 3], 22, consts[ 4])
	a = z(f, a, b, c, d, X[ 4],  7, consts[ 5])
	d = z(f, d, a, b, c, X[ 5], 12, consts[ 6])
	c = z(f, c, d, a, b, X[ 6], 17, consts[ 7])
	b = z(f, b, c, d, a, X[ 7], 22, consts[ 8])
	a = z(f, a, b, c, d, X[ 8],  7, consts[ 9])
	d = z(f, d, a, b, c, X[ 9], 12, consts[10])
	c = z(f, c, d, a, b, X[10], 17, consts[11])
	b = z(f, b, c, d, a, X[11], 22, consts[12])
	a = z(f, a, b, c, d, X[12],  7, consts[13])
	d = z(f, d, a, b, c, X[13], 12, consts[14])
	c = z(f, c, d, a, b, X[14], 17, consts[15])
	b = z(f, b, c, d, a, X[15], 22, consts[16])

	a = z(g, a, b, c, d, X[ 1],  5, consts[17])
	d = z(g, d, a, b, c, X[ 6],  9, consts[18])
	c = z(g, c, d, a, b, X[11], 14, consts[19])
	b = z(g, b, c, d, a, X[ 0], 20, consts[20])
	a = z(g, a, b, c, d, X[ 5],  5, consts[21])
	d = z(g, d, a, b, c, X[10],  9, consts[22])
	c = z(g, c, d, a, b, X[15], 14, consts[23])
	b = z(g, b, c, d, a, X[ 4], 20, consts[24])
	a = z(g, a, b, c, d, X[ 9],  5, consts[25])
	d = z(g, d, a, b, c, X[14],  9, consts[26])
	c = z(g, c, d, a, b, X[ 3], 14, consts[27])
	b = z(g, b, c, d, a, X[ 8], 20, consts[28])
	a = z(g, a, b, c, d, X[13],  5, consts[29])
	d = z(g, d, a, b, c, X[ 2],  9, consts[30])
	c = z(g, c, d, a, b, X[ 7], 14, consts[31])
	b = z(g, b, c, d, a, X[12], 20, consts[32])

	a = z(h, a, b, c, d, X[ 5],  4, consts[33])
	d = z(h, d, a, b, c, X[ 8], 11, consts[34])
	c = z(h, c, d, a, b, X[11], 16, consts[35])
	b = z(h, b, c, d, a, X[14], 23, consts[36])
	a = z(h, a, b, c, d, X[ 1],  4, consts[37])
	d = z(h, d, a, b, c, X[ 4], 11, consts[38])
	c = z(h, c, d, a, b, X[ 7], 16, consts[39])
	b = z(h, b, c, d, a, X[10], 23, consts[40])
	a = z(h, a, b, c, d, X[13],  4, consts[41])
	d = z(h, d, a, b, c, X[ 0], 11, consts[42])
	c = z(h, c, d, a, b, X[ 3], 16, consts[43])
	b = z(h, b, c, d, a, X[ 6], 23, consts[44])
	a = z(h, a, b, c, d, X[ 9],  4, consts[45])
	d = z(h, d, a, b, c, X[12], 11, consts[46])
	c = z(h, c, d, a, b, X[15], 16, consts[47])
	b = z(h, b, c, d, a, X[ 2], 23, consts[48])

	a = z(i, a, b, c, d, X[ 0],  6, consts[49])
	d = z(i, d, a, b, c, X[ 7], 10, consts[50])
	c = z(i, c, d, a, b, X[14], 15, consts[51])
	b = z(i, b, c, d, a, X[ 5], 21, consts[52])
	a = z(i, a, b, c, d, X[12],  6, consts[53])
	d = z(i, d, a, b, c, X[ 3], 10, consts[54])
	c = z(i, c, d, a, b, X[10], 15, consts[55])
	b = z(i, b, c, d, a, X[ 1], 21, consts[56])
	a = z(i, a, b, c, d, X[ 8],  6, consts[57])
	d = z(i, d, a, b, c, X[15], 10, consts[58])
	c = z(i, c, d, a, b, X[ 6], 15, consts[59])
	b = z(i, b, c, d, a, X[13], 21, consts[60])
	a = z(i, a, b, c, d, X[ 4],  6, consts[61])
	d = z(i, d, a, b, c, X[11], 10, consts[62])
	c = z(i, c, d, a, b, X[ 2], 15, consts[63])
	b = z(i, b, c, d, a, X[ 9], 21, consts[64])

	return
		bit.band(A + a, ff),
		bit.band(B + b, ff),
		bit.band(C + c, ff),
		bit.band(D + d, ff)
end

local function LIntToString(i)
	local f = function (s)
		return string.char(bit.band(bit.rshift(i, s), 255))
	end

	return f(0) .. f(8) .. f(16) .. f(24)
end

local function StringToBInt(s)
	local v = 0
	for i = 1, string.len(s) do
		v = v * 256 + string.byte(s, i)
	end
	return v
end

local function StringToLInt(s)
	local v = 0
	for i = string.len(s), 1, -1 do
		v = v * 256 + string.byte(s, i)
	end
	return v
end

local function StringToLIntArray(s, size, count)
	local o, r = 1, {}

	for i = 1, count do
		table.insert(r, StringToLInt(string.sub(s, o, o + size - 1)))
		o = o + size
	end

	return r
end

function lib:MD5AsTable(s)
	local l = string.len(s)
	local pad = 56 - bit.mod(l, 64)

	if bit.mod(l, 64) > 56 then
		pad = pad + 64
	end
	if pad == 0 then
		pad = 64
	end

	s = s .. string.char(128) .. string.rep(string.char(0), pad - 1)
	s = s .. LIntToString(8 * l) .. LIntToString(0)

	assert(bit.mod(string.len(s), 64) == 0)

	local a, b, c, d = consts[65], consts[66], consts[67], consts[68]

	for i = 1, string.len(s), 64 do
		local x = StringToLIntArray(string.sub(s, i, i + 63), 4, 16)

		assert(#(x) == 16)
		x[0] = table.remove(x, 1)

		a, b, c, d = MD5Transform(x, a, b, c, d)
	end
 
	local swap = function (w)
		return StringToBInt(LIntToString(w))
	end

	return {
		swap(a),
		swap(b),
		swap(c),
		swap(d),
	}
end

function lib:MD5(s)
	local h = self:MD5AsTable(s)

	return string.format("%08x%08x%08x%08x", h[1], h[2], h[3], h[4])
end

do
	local s0 = 'message digest'
	local s1 = 'abcdefghijklmnopqrstuvwxyz'
	local s2 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789'
	local s3 = '12345678901234567890123456789012345678901234567890123456789012345678901234567890'

	assert(lib:MD5('') == 'd41d8cd98f00b204e9800998ecf8427e')
	assert(lib:MD5(s0) == 'f96b697d7cb7938d525a2f31aaf161d0')
	assert(lib:MD5(s1) == 'c3fcd3d76192e4007dfb496cca67e13b')
	assert(lib:MD5(s2) == 'd174ab98d277d9f5a5611c2c9f419d9f')
	assert(lib:MD5(s3) == '57edf4a22be3c955ac49da2e2107b67a')
end

AceLibrary:Register(lib, major, minor)

